/*
   Copyright (C) 2020-2023  Gleb Fotengauer-Malinovskiy <glebfm@altlinux.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
   02110-1301, USA.  */

#include "sysdep.h"

#ifdef HAVE_LIBDEBUGINFOD
# include <dlfcn.h>
# include <elfutils/debuginfod.h>
# include "debuginfod.h"

__typeof__ (debuginfod_begin) *fp_debuginfod_begin;
__typeof__ (debuginfod_find_debuginfo) *fp_debuginfod_find_debuginfo;
__typeof__ (debuginfod_find_source) *fp_debuginfod_find_source;
__typeof__ (debuginfod_end) *fp_debuginfod_end;

int
binutils_load_debuginfod (void)
{
  static int debuginfod_loaded = -1;

  if (debuginfod_loaded >= 0)
    return debuginfod_loaded;

  debuginfod_loaded = 0;

  char *urls = getenv (DEBUGINFOD_URLS_ENV_VAR);
  if (urls == NULL || urls[0] == '\0')
    return debuginfod_loaded;

  void *debuginfod_so = dlopen (DEBUGINFOD_SONAME, RTLD_LAZY);
  if (debuginfod_so != NULL)
    {
      fp_debuginfod_begin = dlsym (debuginfod_so, "debuginfod_begin");
      fp_debuginfod_find_debuginfo = dlsym (debuginfod_so,
					    "debuginfod_find_debuginfo");
      fp_debuginfod_find_source = dlsym (debuginfod_so,
					 "debuginfod_find_source");
      fp_debuginfod_end = dlsym (debuginfod_so, "debuginfod_end");

      /* We either get them all, or we get none.  */
      if (fp_debuginfod_begin == NULL
	  || fp_debuginfod_find_debuginfo == NULL
	  || fp_debuginfod_find_source == NULL
	  || fp_debuginfod_end == NULL)
	{
	  fp_debuginfod_begin = NULL;
	  fp_debuginfod_find_debuginfo = NULL;
	  fp_debuginfod_find_source = NULL;
	  fp_debuginfod_end = NULL;
	  dlclose (debuginfod_so);
	}
      else
	debuginfod_loaded = 1;
    }
  return debuginfod_loaded;
}
#endif
